/****************************************************************************/
/* Copyright(C)2006 Tony Bayart                                             */
/* http://www.developpez.net/forums/member.php?u=59049                      */
/* http://shenron.free.fr                                                   */
/*                                                                          */
/* This program is free software; you can redistribute it and/or modify it  */
/* under the terms of the GNU General Public License as published by the    */
/* Free Software Foundation; either version 2 of the License, or            */
/* (at your option) any later version.                                      */
/*                                                                          */
/* This program is distributed in the hope that it will be useful,          */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     */
/* See the GNU General Public License for more details.                     */
/*                                                                          */
/* You should have received a copy of the GNU General Public License along  */
/* with this program; if not, write to the Free Software Foundation, Inc.,  */
/* 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            */
/****************************************************************************/


#include <sdl/SDL.h>
#include <gl/gl.h>
#include <gl/glu.h>


bool Initialize(void)
{
    // la taille de notre fentre
    int ScreenWidth = 640;
    int ScreenHeight = 480;

    // initialisation SDL video
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        printf( "Echec SDL_Init : %s\n", SDL_GetError() );
        return false;
    }

    // SDL initialise, on s'assure que SDL_Quit sera excut avant de quitter
    atexit(SDL_Quit);

    // activation du double buffer OpenGL
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

    // cration de la fentre
    SDL_Surface* screen = SDL_SetVideoMode(ScreenWidth, ScreenHeight, 32, SDL_OPENGL);
    if ( !screen )
    {
        printf("Echec de cration de la fentre en %dx%d : %s\n", ScreenWidth, ScreenHeight, SDL_GetError());
        return false;
    }

    // pour le style, on donne un titre  notre fentre
    SDL_WM_SetCaption("Exemples de formes OpenGL", NULL);

    // on paramtre la couleur d'effacement de l'cran  noir
    glClearColor(0, 0, 0, 0);

    // initialisation de l'affichage OpenGL
    glViewport(0, 0, ScreenWidth, ScreenHeight);

    // slection de la matrice de projection
    glMatrixMode(GL_PROJECTION);

    // reset de la matrice
    glLoadIdentity();

    // configuration de la transformation en perspective
    gluPerspective(45., (float)ScreenWidth / (float)ScreenHeight, 0.1, 100.);

    // slection de la matrice de transformation 3D
    glMatrixMode(GL_MODELVIEW);

    // paisseur des lignes
    glLineWidth(2.f);

    return true;
}

void DrawFormes(void)
{
    float xt = 2.1f;
    float yt = 1.2f;
    float scale = 0.8f;

    glPushMatrix();
    glTranslatef(-xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone concave
        glVertex3f( -1,  -1,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f(  1,  -1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(-xt, -yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   0,  0);
        glVertex3f(-.5,   1,  0);
        glVertex3f( .5,   1,  0);
        glVertex3f(  1,   0,  0);
        glVertex3f( .5,  -1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(   0,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_QUADS); // 1 quadrilatre concave
        glVertex3f( -1,  -1,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
        glVertex3f(-.5,  .5,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(   0, -yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_QUADS); // 1 quadrilatre convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
        glVertex3f( .5,  -1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLES); // 2 triangles, mme forme que le quadrilatre concave
        glVertex3f( -1,  -1,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(-.5,  .5,  0);
        glVertex3f(-.5,  .5,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( xt, -yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLES); // 4 triangles, mme forme que le polygone concave
        glVertex3f(-.5,   0,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f( -1,  -1,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
    glEnd();
    glPopMatrix();
}

void DrawFan(void)
{
    float xt = 2.1f;
    float yt = 1.2f;
    float scale = 0.8f;

    glPushMatrix();
    glTranslatef(-xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   0,  0);
        glVertex3f(-.5,   1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(  0,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   0,  0);
        glVertex3f(-.5,   1,  0);
        glVertex3f( .5,   1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(  xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   0,  0);
        glVertex3f(-.5,   1,  0);
        glVertex3f( .5,   1,  0);
        glVertex3f(  1,   0,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(   0, -yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_FAN); // polygone convexe
        glVertex3f(-.5,  -1,  0);
        glVertex3f( -1,   0,  0);
        glVertex3f(-.5,   1,  0);
        glVertex3f( .5,   1,  0);
        glVertex3f(  1,   0,  0);
        glVertex3f( .5,  -1,  0);
    glEnd();
    glPopMatrix();
}

void DrawStrip(void)
{
    float xt = 2.1f;
    float yt = 1.2f;
    float scale = 0.8f;

    glPushMatrix();
    glTranslatef(-xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_STRIP); // polygone convexe
        glVertex3f( -1,  -1,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f(-.5,   0,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(  0,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_STRIP); // polygone convexe
        glVertex3f( -1,  -1,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f( .5,   0,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(  xt,  yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_STRIP); // polygone convexe
        glVertex3f( -1,  -1,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f( -1,   1,  0);
    glEnd();
    glPopMatrix();

    glPushMatrix();
    glTranslatef(   0, -yt, 0);
    glScalef(scale, scale, scale);
    glBegin(GL_TRIANGLE_STRIP); // polygone convexe
        glVertex3f( -1,  -1,  0);
        glVertex3f(  1,  -1,  0);
        glVertex3f(-.5,   0,  0);
        glVertex3f( .5,   0,  0);
        glVertex3f( -1,   1,  0);
        glVertex3f(  1,   1,  0);
    glEnd();
    glPopMatrix();
}

void DrawFrame(int choix)
{
    // effacement de l'cran
    glClear(GL_COLOR_BUFFER_BIT);

    // (r)initialisation de la matrice de transformation
    glLoadIdentity();

    // on dplace tout vers le fond
    glTranslatef(0, 0, -6);

    // affichage des polygones en mode remplissage
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glColor3ub(  0,   0, 255); // couleur : bleu
    switch(choix)
    {
        case 0:
            DrawFormes();
            break;
        case 1:
            DrawFan();
            break;
        case 2:
            DrawStrip();
            break;
        default:
            DrawFormes();
            break;
    }

    // affichage des polygones en mode lignes (contours)
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glColor3ub(255, 255, 255); // couleur : blanc
    switch(choix)
    {
        case 0:
            DrawFormes();
            break;
        case 1:
            DrawFan();
            break;
        case 2:
            DrawStrip();
            break;
        default:
            DrawFormes();
            break;
    }
}


int main ( int argc, char** argv )
{
    if( !Initialize() )
        return 1;

    int choix = 0;
    bool bRunning = true;
    // boucle principale du programme
    while (bRunning)
    {
        SDL_Event event; // un vnement SDL
        // rcupration d'un vnement dans la pile
        while (SDL_PollEvent(&event))
        {
            // analyse du message
            switch (event.type)
            {
                // message demande  quitter
                case SDL_QUIT:
                    bRunning = false;
                    break;
                // message signalant l'appui sur une touche
                case SDL_KEYDOWN:
                    // exit if ESCAPE is pressed
                    if (event.key.keysym.sym == SDLK_ESCAPE)
                        bRunning = false;
                    else if(event.key.keysym.sym == SDLK_1) // appuyez sur la touche 1 pour voir les primitives
                        choix = 0;
                    else if(event.key.keysym.sym == SDLK_2) // appuyez sur la touche 2 pour voir le triangle fan
                        choix = 1;
                    else if(event.key.keysym.sym == SDLK_3) // appuyez sur la touche 2 pour voir le triangle strip
                        choix = 2;
                    break;
            } // fin du switch
        } // fin de la boucle de gestion des vnements

        // dessin
        DrawFrame(choix);

        // change des buffers back et front
        SDL_GL_SwapBuffers();
    } // fin de la boucle principale du programme

    printf("Fin normale du programme\n");

    return 0;
}
